using System;
using System.Web.UI;
using System.Web;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.Web.UI.Design;

namespace Interval.LoginTemplateControl
{
	/// <summary>
	/// Serverov ovldac prvek, kter dovoluje definovat odlin ablony pro anonymnho a pihlenho uivatele
	/// </summary>
	[DefaultProperty("AnonymousUserMessage")]
	[DefaultEvent("LoginRequest")]
	[ToolboxData("<{0}:LoginControl runat=server></{0}:LoginControl>")]
	[Designer(typeof(LoginControlDesigner))]
	public class LoginControl : System.Web.UI.WebControls.WebControl, INamingContainer
	{
		

		#region Delegates
		public delegate void LoginItemCommandEventHandler (object sender, CommandEventArgs e);
		#endregion Delegates
		#region Public Constants
		/// <summary>
		/// popisek na tlatko pro udlost <see cref="LoginRequest"/>
		/// </summary>
		public const string LOGIN_BUTTON_NAME = "Login";
		
		/// <summary>
		/// Konstanta reprezentuje popisek na tlatko pro udlost <see cref="LogoutRequest"/>
		/// </summary>
		public const string LOGOUT_BUTTON_NAME = "Logout";
		
		#endregion Public Constants

		#region Events Keys

		/// <summary>
		/// Kl udlosti <see cref="LoginRequest"/>
		/// </summary>
		public static readonly Object LoginRequestKey = new Object();
		
		/// <summary>
		/// Kl udlosti <see cref="LoginRequest"/>
		/// </summary>
		public static readonly Object LogoutRequestKey = new Object();
		
		/// <summary>
		/// Kl udlosti <see cref="ItemCommand"/>
		/// </summary>
		public static readonly Object ItemCommandKey = new Object();
		
		#endregion Events Keys
		

		#region Public Events
		/// <summary>
		/// Udlost 'Pihlsit uivatele' je vyvolna po kliknutu na tlatko s popiskem <see cref="LoginButtonName"/>
		/// </summary>
		public event EventHandler LoginRequest
		{
			add
			{
				Events.AddHandler(LoginRequestKey, value);
			}
			
			remove
			{
				Events.RemoveHandler(LoginRequestKey, value);
			}
		}
		
		/// <summary>
		/// Udlost 'Odhlsit  uivatele' je vyvolna po kliknutu na tlatko s popiskem <see cref="LogoutButtonName"/>
		/// </summary>
		public event EventHandler LogoutRequest
		{
			add
			{
				Events.AddHandler(LogoutRequestKey, value);
			}
			
			remove
			{
				Events.RemoveHandler(LogoutRequestKey, value);
			}
		}
		
		/// <summary>
		/// Udlost zprostedkovv udlosti vnoench ovldacch prvk
		/// </summary>
		public event LoginItemCommandEventHandler ItemCommand
		{
			add
			{
				Events.AddHandler(ItemCommandKey, value);
			}
			
			remove
			{
				Events.RemoveHandler(ItemCommandKey, value);
			}
		}
		
		#endregion Public Events
		
		
		#region Private variables
		private LoginControlContent m_content;
		private ITemplate m_anonymousTemplate;
		private ITemplate m_loggedInTemplate;
		#endregion Private variables
		
		#region Contructors
		/// <summary>
		/// Konstruktor
		/// </summary>
		public LoginControl()
		{

		}
		#endregion Contructors
		
		
		#region Public properties
		public override ControlCollection Controls
		{
			get
			{
				EnsureChildControls();
				return base.Controls;
			}
		}


		/// <summary>
		/// Vlastnost dovoluje pistupovat k prvkm ablony pes FindControl
		/// </summary>
		[Browsable(false)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public LoginControlContent Content
		{
			get
			{
				EnsureChildControls();
				return m_content;
			}
		}

		/// <summary>
		/// ablona pro pihlenho uivatele
		/// </summary>
		[Browsable(false)]
		[DefaultValue(null)]
		[PersistenceMode(PersistenceMode.InnerProperty)]
		[TemplateContainer(typeof(LoginControlContent))]
		public ITemplate LoggedInTemplate
		{
			get
			{
				return m_loggedInTemplate;
			}

			set
			{
				m_loggedInTemplate = value;
			}
		}

		/// <summary>
		/// ablona pro nepihlenho uivatele
		/// </summary>
		[Browsable(false)]
		[DefaultValue(null)]
		[PersistenceMode(PersistenceMode.InnerProperty)]
		[TemplateContainer(typeof(LoginControlContent))]
		public ITemplate AnonymousTemplate
		{
			get
			{
				return m_anonymousTemplate;
			}

			set
			{
				m_anonymousTemplate = value;
			}
		}
		
		/// <summary>
		/// Informace zobrazen nepihlenmu uivateli
		/// </summary>
		[Category("Behavior")]
		[Bindable(true)]
		[DefaultValue("")]
		[Description("Informace zobrazen nepihlenmu uivateli")]
		public string AnonymousUserMessage
		{
			get
			{
				string message = (string) ViewState["AnonymousUserMessage"];
				return (message == null ? String.Empty : message);
			}
			
			set
			{
				ViewState["AnonymousUserMessage"] = value;
			}
		}

		/// <summary>
		/// Informace zobrazen pihlenmu uivateli
		/// </summary>
		[Category("Behavior")]
		[Bindable(true)]
		[DefaultValue("")]
		[Description("Informace zobrazen pihlenmu uivateli")]
		public string LoggedInMessage
		{
			get
			{
				string message = (string) ViewState["LoggedInMessage"];
				return (message == null ? String.Empty : message);
			}
			set
			{
				ViewState["LoggedInMessage"] = value;
			}
		}
		#endregion Public properties		
		
		#region Protected properties		
		
		/// <summary>
		/// Prvek bude uzaven v HTML znace DIV
		/// </summary>
		protected override HtmlTextWriterTag TagKey
		{
			get
			{
				return HtmlTextWriterTag.Div;
			}
		}

		#endregion Protected properties		

		#region Public methods
		
		/// <summary>
		/// Pepsn metody pro vyhodnocovn datov orientovanch vraz
		/// </summary>
		public override void DataBind()
		{
			Controls.Clear();
			CreateChildControls();
			ChildControlsCreated = true;
			base.DataBind ();
		}

		#endregion Public methods
		
		#region Protected methods
		
	
		/// <summary>
		/// Vytvoen vnoench ovldacch prvk
		/// </summary>
		protected override void CreateChildControls()
		{
			
			Controls.Clear();
			
			
				if  ((HttpContext.Current != null) && (Page != null) && (Page.User != null) && (Page.User.Identity.IsAuthenticated))
				{
					m_content = new LoginControlContent(Page.User.Identity.Name, this.AnonymousUserMessage, this.LoggedInMessage);
					ITemplate template = null;
				
					if (m_loggedInTemplate != null)
						template = m_loggedInTemplate;
					else
						template = new DefaultLoggedInTemplate();
				
					template.InstantiateIn(m_content);
				
				}

			

				else
				{
					m_content = new LoginControlContent(String.Empty, this.AnonymousUserMessage, this.LoggedInMessage);
				
					ITemplate template = null;
				
					if (m_anonymousTemplate != null)
						template = m_anonymousTemplate;
					else
						template = new DefaultAnonymousTemplate();
				
					template.InstantiateIn(m_content);
					
				
				}
			
			this.Controls.Add(m_content);
			
		}

		/// <summary>
		/// Ppsn metody, kter zachycuje bublan udlosti
		/// </summary>
		/// <param name="source">Zdroj bublan udlosti</param>
		/// <param name="args">Argumenty bublan udlosti</param>
		/// <returns><see cref="true"/> - Udlost byla zpracovna, <see cref="false"/> - Udlost nebyla zpracovna</returns>
		protected override bool OnBubbleEvent(object source, EventArgs e)
		{
			CommandEventArgs ex = e as CommandEventArgs;
			if (ex != null)
			{
				if (ex.CommandName.ToLower() == LOGIN_BUTTON_NAME.ToLower())
					OnLoginRequest(new EventArgs());
				
				else if (ex.CommandName.ToLower() == LOGOUT_BUTTON_NAME.ToLower())
					OnLogoutRequest(new EventArgs());
				
				else 
					OnItemCommand(ex);
				
				return true;
			}
			
			return false;
		}

		/// <summary>
		/// Metoda odpovdn za vyvoln udlosti loginRequest
		/// </summary>
		/// <param name="e">Parametry udlosti</param>
		protected virtual void OnLoginRequest(EventArgs e)
		{
			EventHandler eh = (EventHandler) Events[LoginRequestKey];
			if (eh != null)
				eh(this, e);
		}

		/// <summary>
		/// Metoda odpovdn za vyvoln udlosti LogoutRequest
		/// </summary>
		/// <param name="e">Parametry udlosti</param>
		protected virtual void OnLogoutRequest(EventArgs e)
		{
			EventHandler eh = (EventHandler) Events[LogoutRequestKey];
			if (eh != null)
				eh(this, e);
		}

		/// <summary>
		/// Metoda odpovdn za vyvoln udlosti ItemCommand
		/// </summary>
		/// <param name="e">Parametry udlosti</param>
		protected virtual void OnItemCommand(CommandEventArgs e)
		{
			LoginItemCommandEventHandler eh = (LoginItemCommandEventHandler) Events[ItemCommandKey];
			if (eh != null)
				eh(this, e);
		}

		#endregion Protected methods
	}
}
